<?php

namespace Drupal\Tests\vmware\Traits;

use Drupal\Component\Utility\Random;

/**
 * The trait creating form data for vmware testing.
 */
trait VmwareTestFormDataTrait {

  /**
   * Create test data for cloud service provider (CloudConfig).
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   Test data.
   */
  protected function createCloudConfigTestFormData($repeat_count): array {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0, $num = 1; $i < $repeat_count; $i++, $num++) {

      $data[] = [
        'name[0][value]'                   => sprintf('config-entity-#%d-%s - %s', $num, $random->name(8, TRUE), date('Y/m/d H:i:s')),
        'cloud_context'                    => strtolower($random->name(16, TRUE)),
        'field_vcenter_url[0][value]'      => 'http://example.com/',
        'field_vcenter_username[0][value]' => $random->name(16, TRUE),
        'field_vcenter_password[0][value]' => $random->name(16, TRUE),
      ];
    }

    return $data;
  }

  /**
   * Create test data for VMware VM.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   Test data.
   */
  protected function createVmTestFormData($repeat_count): array {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 1; $i <= $repeat_count; $i++) {
      $data[] = [
        'name' => sprintf('VM-%s - %s', $random->name(8, TRUE), date('Y/m/d H:i:s')),
        'host' => $this->latestTemplateVars["host$i"],
        'folder' => $this->latestTemplateVars["folder$i"],
        'datastore' => $this->latestTemplateVars["datastore$i"],
        'guest_os' => 'DOS',
      ];
    }

    return $data;
  }

  /**
   * Create random VMs data.
   *
   * @return array
   *   Random VMs.
   *
   * @throws \Exception
   */
  protected function createVmsRandomTestFormData(): array {
    $vms = [];
    $count = random_int(1, 10);
    for ($i = 0, $num = 1; $i < $count; $i++, $num++) {
      $vms[] = [
        // 'cloud_context' needs to associate a fixed $cloud_context since the
        // workspace entities belong to a specific $cloud_context.
        // The Terraform organization is equal to a cloud service provider
        // (CloudConfig), which is $cloud_context here.
        'cloud_context' => $this->cloudContext,
        'name' => sprintf('vm #%d - %s - %s', $num, date('Y/m/d H:i:s'), $this->random->name(4, TRUE)),
      ];
    }

    return $vms;
  }

}
