<?php

namespace Drupal\Tests\vmware\Traits;

use Drupal\Component\Utility\Random;

/**
 * The trait creating mock data for vmware testing.
 */
trait VmwareTestMockTrait {

  /**
   * Update createVm in mock data.
   *
   * @param array $vm
   *   The VM mock data.
   */
  protected function addVmMockData(array $vm): void {
    $mock_data = $this->getMockDataFromConfig();
    $random = new Random();
    $vm_id = 'vm-' . rand(1, 100);
    $create_vm = [
      'value' => $vm_id,
    ];

    $mock_data['createVm'] = $create_vm;
    $mock_data['describeVms'] = [
      'value' => [
        [
          'vm' => $vm_id,
          'name' => $vm['name'],
          'power_state' => 'POWERED_OFF',
          'cpu_count' => 1,
          'memory_size_MiB' => 1024,
          'cdroms' => [
            [
              'value' => [
                'state' => 'CONNECTED',
                'backing' => [
                  'iso_file' => 'isofile',
                ],
                'label' => 'cdrom - ' . $random->name(8, TRUE),
              ],
            ],
          ],
          'networks' => [
            'value' => [
              'state' => 'CONNECTED',
              'label' => 'network - ' . $random->name(8, TRUE),
            ],
          ],
        ],
      ],
    ];

    $mock_data['showVm'] = [
      'value' => [
        'guest_OS' => 'UBUNTU_64',
        'disks' => [],
        'cdroms' => [
          [
            'value' => [
              'state' => 'CONNECTED',
              'backing' => [
                'iso_file' => 'isofile',
              ],
              'label' => 'cdrom - ' . $random->name(8, TRUE),
            ],
          ],
        ],
        'nics' => [
          [
            'value' => [
              'state' => 'CONNECTED',
              'label' => 'network - ' . $random->name(8, TRUE),
              'backing' => [
                'network_name' => 'VM NETWORK - ' . $random->name(8, TRUE),
              ],
            ],
          ],
        ],
      ],
    ];

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update deleteVm in mock data.
   *
   * @param array $vm
   *   The VM mock data.
   */
  protected function deleteVmMockData(array $vm): void {
    $mock_data = $this->getMockDataFromConfig();

    $delete_vm = [];
    $mock_data['deleteVm'] = $delete_vm;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update the power state of VM in mock data.
   *
   * @param string $power_state
   *   The power state of VM.
   */
  protected function updateVmPowerStateInMockData($power_state): void {
    $mock_data = $this->getMockDataFromConfig();

    $mock_data['describeVms']['value'][0]['power_state'] = $power_state;
    $this->updateMockDataToConfig($mock_data);
  }

}
