<?php

namespace Drupal\cloud\Traits;

/**
 * The trait with common functions for Resource Blocks.
 */
trait ResourceBlockTrait {

  use CloudContentEntityTrait;

  /**
   * Get a count of all aws resources.
   *
   * @param string $resource_name
   *   The resource name.
   * @param string $permission
   *   The permission.
   * @param array $params
   *   The params.
   *
   * @return int|void
   *   Entity count.
   */
  protected function getResourceCount($resource_name, $permission, array $params) {
    if (!$this->currentUser->hasPermission($permission)) {
      $params['uid'] = $this->currentUser->id();
    }
    $entities = $this->runEntityQuery($resource_name, $params);
    return count($entities);
  }

  /**
   * Execute an entity query.
   *
   * @param string $entity_name
   *   The entity name.
   * @param array $params
   *   Array of parameters.
   *
   * @return \Drupal\Core\Entity\EntityInterface[]
   *   An array of loaded entities.
   */
  protected function runEntityQuery($entity_name, array $params) {
    $cloud_context = $this->configuration['cloud_context'];
    /** @var \Drupal\Core\Routing\RouteMatchInterface $route_match */
    $cloud_project = \Drupal::routeMatch()->getParameter('cloud_project');
    if (empty($cloud_project) && !empty($cloud_context)) {
      $params['cloud_context'] = $cloud_context;
    }
    return $this->entityTypeManager->getStorage($entity_name)
      ->loadByProperties($params);
  }

  /**
   * Load the cloud configs as an array for use in a select dropdown.
   *
   * @param string $default_text
   *   The default dropdown option.
   * @param string $entity_type
   *   The entity type to load.
   *
   * @return array
   *   An array of cloud configs.
   */
  protected function getCloudConfigs($default_text, $entity_type) {
    $cloud_configs = ['' => $default_text];
    $configs = $this->cloudConfigPluginManager->loadConfigEntities($entity_type);
    foreach ($configs ?: [] as $config) {
      $cloud_configs[$config->getCloudContext()] = $config->getName();
    }
    return $cloud_configs;
  }

}
